@@ -1,7 +1,9 @@
 use std::fs;
 use std::path::{Path, PathBuf};
+use mysql::{Opts, OptsBuilder};
 
 use super::restriction::{Restriction};
+use super::super::process_connection;
 
 /// Possible dot output formats.
 pub const POSSIBLE_DOTS_OUTPUT : [&str; 54] = ["bmp", "canon", "gv", "xdot", "xdot1.2", "xdot1.4",
@@ -18,7 +20,9 @@ pub const POSSIBLE_DOTS_OUTPUT : [&str; 54] = ["bmp", "canon", "gv", "xdot", "xd
 #[derive(Clone)]
 pub struct Args {
     /// Filename
-    filename: String,
+    filename: Option<String>,
+    /// Connection options for database
+    opts : Option<Opts>,
     /// File content
     filecontent: String,
     /// Output file name
@@ -37,42 +41,100 @@ impl Args {
     ///
     /// # Arguments
     ///
-    /// * `path_str` - The path of the input file
-    pub fn new(path_str: Vec<&str>) -> Args {
+    /// * `path_str` - The path of the input
+    pub fn new_from_files(path_str: Vec<&str>) -> Result<Args, Box<dyn std::error::Error>> {
         let filename : String;
         if path_str.len() != 1 {
             filename = String::from("multifilearg");
         } else {
-            filename = Path::new(path_str.first().unwrap()).file_name().expect("Incorrect file name").to_str().unwrap().to_string();
+            filename = match Path::new(path_str[0]).file_name() {
+                Some(v) => v.to_str().unwrap().to_string(),
+                None => return Err("No file found for given input".into()),
+            };
         }
-        Args {
-            filename,
-            filecontent: path_str.iter().map(|path| {
-                if Path::new(path).is_dir() {
-                    fs::read_dir(path).expect("Directory can't be read").map(|file|
-                        {
-                            let file_path : &PathBuf = &file.unwrap().path();
-                            if Path::new(file_path).is_file() {
-                                fs::read_to_string(file_path).unwrap_or_else(|_| panic!("Something went wrong while reading the file : {}", file_path.as_path().to_str().unwrap_or("**ISSUE**")))
-                            } else {
-                                String::new()
-                            }
-                        }).collect::<Vec<String>>().join("\n")
-                } else {
-                    fs::read_to_string(path).unwrap_or_else(|_| panic!("Something went wrong while reading the file : {}", path))
+        let mut filecontent : Vec<String> = vec![];
+        for path in path_str.iter() {
+            if Path::new(path).is_dir() {
+                for subpath in fs::read_dir(path)? {
+                        let file_path : &PathBuf = &subpath.unwrap().path();
+                        // Ignore subdirs
+                        if Path::new(file_path).is_file() {
+                            filecontent.push(fs::read_to_string(file_path)?);
+                        }
                 }
-            }).collect::<Vec<String>>().join("\n"),
+            } else {
+                filecontent.push(fs::read_to_string(path)?);
+            }
+        }
+        Ok(Args {
+            filename : Some(filename),
+            filecontent: filecontent.join("\n"),
             output_filename: String::from("output.dot"),
+            opts : None,
             restrictions: None,
             legend: false,
             dark_mode: false
-        }
+        })
     }
 
 
+    /// Returns a args object for the given filename
+    ///
+    /// # Arguments
+    ///
+    /// * `url` - The path of the input
+    pub fn new_from_url(url: &str) -> Result<Args, mysql::Error> {
+        let opts : Opts = Opts::from_url(url)?;
+        let mut args : Args =
+            Args {
+                filename : None,
+                filecontent: String::new(),
+                output_filename: String::from("output.dot"),
+                opts: Some(opts),
+                restrictions: None,
+                legend: false,
+                dark_mode: false
+            };
+        process_connection(&mut args)?;
+        Ok(args)
+    }
+
+    /// Returns a args object for the given parameters
+    ///
+    /// # Arguments
+    /// 
+    /// * `db_url` - Database url or ip
+    /// * `db_port` - Database remote port
+    /// * `db_name` - Database remote schema name
+    /// * `db_user` - Database remote user
+    /// * `db_password` - Database remote user's password
+    pub fn new_connect_with_params(db_url: String, db_port: u16, db_name: String, db_user: String, db_password: String) -> Result<Args, mysql::Error> {
+        let opts_builder : OptsBuilder = 
+            OptsBuilder::new().ip_or_hostname(Some(db_url))
+                              .tcp_port(db_port)
+                              .db_name(Some(db_name))
+                              .user(Some(db_user))
+                              .pass(Some(db_password));
+        let mut args : Args =
+        Args {
+            filename : None,
+            filecontent: String::new(),
+            output_filename: String::from("output.dot"),
+            opts: Some(Opts::from(opts_builder)),
+            restrictions: None,
+            legend: false,
+            dark_mode: false
+        };
+        process_connection(&mut args)?;
+        Ok(args)
+    }
+
     /// Returns the filename without the non ascii digits and chars
     pub fn get_filename_without_specials(&self) -> String {
-        self.filename.chars().filter(|c| c.is_ascii_alphanumeric() || c.is_ascii_whitespace()).collect::<String>()
+        match &self.filename {
+            Some(v) => v.chars().filter(|c| c.is_ascii_alphanumeric() || c.is_ascii_whitespace()).collect::<String>(),
+            None => "doteur".to_string()
+        }
     }
 
     /// Returns the file extension
@@ -94,6 +156,10 @@ impl Args {
         self.filecontent.as_str()
     }
 
+    pub fn set_filecontent(&mut self, filecontent: String) {
+        self.filecontent = filecontent;
+    }
+
     /// Get the output file name
     pub fn get_output_filename(&self) -> &str {
         self.output_filename.as_str()
@@ -108,6 +174,10 @@ impl Args {
         self.output_filename = output_filename;
     }
 
+    pub fn get_opts(&self) -> Option<&Opts> {
+        self.opts.as_ref()
+    }
+
     /// Get restrictions
     pub fn get_restrictions(&self) -> Option<&Restriction> {
         self.restrictions.as_ref()
@@ -167,37 +237,3 @@ impl Args {
     }
 
 }
-
-#[cfg(test)]
-mod tests {
-
-    use super::*;
-
-    #[test]
-    fn test_file_ext() {
-        assert_eq!({
-            let args = Args::new(vec!["./samples/samplefile3.sql"]);
-            args.clone().get_output_file_ext()
-        }, "dot", "default value");
-
-        assert_eq!({
-            let mut args = Args::new(vec!["./samples/samplefile3.sql", "./samples/samplefile1.sql"]);
-            args.set_output_filename("hello.png".to_string());
-            args.clone().get_output_file_ext()
-        }, "png", "set value with multifile");
-
-        assert_eq!({
-            let mut args = Args::new(vec!["./samples"]);
-            args.set_output_filename("./path/to/file/hello.png".to_string());
-            args.clone().get_output_file_ext()
-        }, "png", "set value");
-    }
-
-    #[test]
-    fn test_file_in_list() {
-        assert!(POSSIBLE_DOTS_OUTPUT.iter().all(|e| Args::ext_supported(e)), "normal use cases");
-        assert!(!Args::ext_supported("dot"), "normal use case, we don't handle the dot files to the graphviz tool");
-        assert!(!Args::ext_supported("file.png"), "normal use case, only the extension should be given");
-    }
-
-}
